;;;; -*- Mode: Lisp -*-
;;;; $Id: usocket-clozure.lisp 822 2010-09-24 10:24:44Z binghe $

(in-package :ccl)

;;; Backport from USOCKET trunk
(defun c_send-for-usocket (sockfd msgptr len flags)
  (ignoring-eintr (check-socket-error (#_send sockfd msgptr len flags))))

(defun send-for-usocket (socket msg size &key offset)
  "Send a UDP packet over a connected socket."
  (let ((fd (socket-device socket)))
    (multiple-value-setq (msg offset) (verify-socket-buffer msg offset size))
    (%stack-block ((bufptr size))
      (%copy-ivector-to-ptr msg offset bufptr 0 size)
      (socket-call socket "send"
	(with-eagain fd :output
	  (c_send-for-usocket fd bufptr size 0))))))

(in-package :usocket)

(defun socket-connect/udp (host port &key element-type timeout deadline nodelay
			   local-host local-port)
  (declare (ignore element-type timeout deadline nodelay))
  (with-mapped-conditions ()
    (let ((mcl-sock
	   (openmcl-socket:make-socket :address-family :internet
				       :type :datagram
				       :local-host (when local-host (host-to-hostname local-host))
				       :local-port local-port
                                       :format :binary)))
      (when (and host port)
	(ccl::inet-connect (ccl::socket-device mcl-sock)
			   (ccl::host-as-inet-host host)
			   (ccl::port-as-inet-port port "udp")))
      (make-datagram-socket mcl-sock))))

;;; Backport from USOCKET trunk
(defmethod socket-send ((usocket datagram-usocket) buffer length &key host port)
  (with-mapped-conditions (usocket)
    (if (and host port)
	(openmcl-socket:send-to (socket usocket) buffer length
				:remote-host (host-to-hbo host)
				:remote-port port)
	(ccl::send-for-usocket (socket usocket) buffer length))))

(defmethod socket-receive ((usocket datagram-usocket) buffer length &key)
  (with-mapped-conditions (usocket)
    (openmcl-socket:receive-from (socket usocket) length :buffer buffer)))
